@@ -5,7 +5,7 @@ import gi
 import pockethernet
 import logging
 
-from pockethernet import WiremapResult, LinkResult, PoEResult
+from pockethernet import WiremapResult, LinkResult, PoEResult, BerResult
 
 import wiremapper.discover
 import wiremapper.wiremap
@@ -212,7 +212,15 @@ class Quicktest(threading.Thread):
             return
 
         if wiremap.shorts == [None, 3, 6, None, 7, 8, None, None, None] and self.all_none(wiremap.connections):
-            GLib.idle_add(self.callback, False, "Found loopback adapter", [wiremap])
+            GLib.idle_add(self.callback, True, "Running BER test")
+            ber = client.get_ber(speed=100, large_packet=True)
+            if ber.errors == 0 and ber.status:
+                ber = client.get_ber(speed=1000, large_packet=True)
+            if ber.errors == 0 and ber.status:
+                ber_status = "BER test successful"
+            else:
+                ber_status = "BER test FAILED"
+            GLib.idle_add(self.callback, False, ber_status, [ber])
             return
 
         if self.all_none(wiremap.connections) and self.all_none(wiremap.shorts):
@@ -244,7 +252,7 @@ class Quicktest(threading.Thread):
             elif sum(poe.pair_volts) > 3:
                 poe_status = "passive PoE ({}v)".format(max(poe.pair_volts))
 
-            GLib.idle_add(self.callback, False, "{} link established, {}".format(link.speed, poe_status), [link, poe])
+            GLib.idle_add(self.callback, False, "{} link established\n{}".format(link.speed, poe_status), [link, poe])
             return
 
         # Running PoE test after failed link because it might be a PoE injector without upstream
@@ -316,8 +324,7 @@ class AppWindow:
     def __init__(self, application):
         self.application = application
         builder = Gtk.Builder()
-        with pkg_resources.path('wiremapper', 'wiremapper.glade') as ui_file:
-            builder.add_from_file(str(ui_file))
+        builder.add_from_resource('/nl/brixit/wiremapper/ui/wiremapper.ui')
         builder.connect_signals(Handler(builder))
 
         window = builder.get_object("main_window")
@@ -467,6 +474,9 @@ class Handler:
                 elif isinstance(block, PoEResult):
                     box = self.make_result_poe(block)
                     self.quicktest_result.pack_start(box, False, False, True)
+                elif isinstance(block, BerResult):
+                    box = self.make_result_ber(block)
+                    self.quicktest_result.pack_start(box, False, False, True)
             self.quicktest_result.show_all()
 
     def on_custom_start_clicked(self, button):
@@ -601,7 +611,7 @@ class Handler:
             grid.set_row_spacing(12)
             grid.set_column_spacing(12)
 
-            grid.attach(Gtk.Label(label="Link partner advertizing", xalign=0), 0, 0, 1, 1)
+            grid.attach(Gtk.Label(label="Link partner", xalign=0), 0, 0, 1, 1)
             grid.attach(Gtk.Label(label="10 Mbps", xalign=1), 0, 1, 1, 1)
             grid.attach(Gtk.Label(label="100 Mbps", xalign=1), 0, 2, 1, 1)
             grid.attach(Gtk.Label(label="1000 Mbps", xalign=1), 0, 3, 1, 1)
@@ -671,6 +681,41 @@ class Handler:
         box.pack_start(frame, False, False, 12)
         return box
 
+    def make_result_ber(self, result):
+        if not isinstance(result, BerResult):
+            return
+
+        box = self.make_result('Bit Error Rate')
+
+        frame_vbox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=8)
+
+        grid = Gtk.Grid()
+        frame_vbox.pack_start(grid, False, False, True)
+
+        grid.set_row_spacing(12)
+        grid.set_column_spacing(12)
+
+        grid.attach(Gtk.Label(label="Packets sent", xalign=0), 0, 0, 1, 1)
+        grid.attach(Gtk.Label(label="Packets received", xalign=0), 0, 1, 1, 1)
+        grid.attach(Gtk.Label(label="Errors", xalign=0), 0, 2, 1, 1)
+
+        sent = "{:n}".format(result.sent)
+        received = "{:n}".format(result.received)
+        errors = "{} ({:.2f}%)".format(result.errors, (result.errors / result.sent) * 100.0)
+
+        grid.attach(Gtk.Label(label=sent, xalign=0), 1, 0, 1, 1)
+        grid.attach(Gtk.Label(label=received, xalign=0), 1, 1, 1, 1)
+        grid.attach(Gtk.Label(label=errors, xalign=0), 1, 2, 1, 1)
+
+        frame = Gtk.Frame()
+        self.add_class(frame, 'view')
+        aligner = Gtk.Alignment()
+        aligner.set_padding(12, 12, 12, 12)
+        aligner.add(frame_vbox)
+        frame.add(aligner)
+        box.pack_start(frame, False, False, 12)
+        return box
+
     def on_custom_slot_list_row_activated(self, listbox, row):
         if isinstance(row.get_child(), Gtk.Image):
             self.add_slot()
@@ -746,6 +791,10 @@ def main():
     # This is to make the Handy module actually loaded to be used in the GtkBuilder
     Handy.Column()
 
+    with pkg_resources.path('wiremapper', 'wiremapper.gresource') as resource_file:
+        resource = Gio.resource_load(str(resource_file))
+        Gio.Resource._register(resource)
+
     app = WiremapperApplication("nl.brixit.wiremapper", Gio.ApplicationFlags.FLAGS_NONE)
     app.run()
 
